#include "gplbp.h"

void change_preferences(GtkWidget *widget, gpointer *data)
{
	GtkWidget *entry;
	GtkWidget *label;
	GtkWidget *table;
	GtkWidget *buttonbox;
	GtkWidget *button;
	GtkTooltips *tooltips;
	FILE *rcfile;
	gchar filename[128];
	struct CurrentEntries *current;

	if(prefwindow != NULL)
	{
		gtk_widget_destroy(prefwindow);
		prefwindow = NULL;
	}

/* Assume that the rc file does exist, unless the program determines otherwise (below) */
	rcfileexists = 1;

/* Get memory for current, the variable that will old rc file settings. */
	current = malloc(sizeof(struct CurrentEntries));
	if (current == NULL)
	{
		fprintf(stderr, "Malloc failed.  Out of memory?\n");
		exit(255);
	}
	memset(current, '\0', sizeof(struct CurrentEntries));
	memset(pref, '\0', sizeof(struct Preferences));

/* Open the rcfile to read stuff from it, create it if it doesn't exist. */
	snprintf(filename, 127, "%s%s", getenv("HOME"), "/.gplbprc");

	rcfile = fopen(filename, "r+");
        if (rcfile == NULL)
        {
                rcfile = fopen(filename, "w+");
		rcfileexists = 0;
                if (rcfile == NULL)
                {
			#ifdef DEBUG
                	 	g_print(stderr, "Creation of %s failed.\n", filename);
				fflush(stdout);
			#endif
                         /* exit(254); */
			rcfileexists = 0;
                }
        }

	if (rcfileexists == 1)
	{
		parse_rc(current);
	}
        #ifdef DEBUG
                g_print("Just opened %s.\n", filename);
                fflush(stdout);
        #endif
	
	
	prefwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);

	table = gtk_table_new(6, 2, FALSE);
	
        label = gtk_label_new("Default Tail Number:");
        entry = gtk_entry_new_with_max_length(TNMAX);
	if (strlen(current->TailNumber) != 0)
        {
		gtk_entry_set_text(GTK_ENTRY(entry), current->TailNumber);
		strncpy(pref->TailNumber, current->TailNumber, TNMAX);
	}
	else
	{
		gtk_entry_set_text(GTK_ENTRY(entry), DEFAULTTAILNUMBER);
	}
        gtk_widget_set_usize(entry, 100, 20);
        gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, 0, 0, 0, 0);
        gtk_table_attach(GTK_TABLE(table), entry, 1, 2, 0, 1, 0, 0, 0, 0);
	gtk_signal_connect(GTK_OBJECT(entry), "changed", GTK_SIGNAL_FUNC(tailprefchanged), NULL);
        gtk_widget_show(label);
        gtk_widget_show(entry);


	label = gtk_label_new("Default Make/Model:");
	entry = gtk_entry_new_with_max_length(MMMAX);
	if(strlen(current->MakeModel) != 0)
        {
                gtk_entry_set_text(GTK_ENTRY(entry), current->MakeModel);
		strncpy(pref->MakeModel, current->MakeModel, MMMAX);
        }
        else
        {
                gtk_entry_set_text(GTK_ENTRY(entry), "Cessna 172");
        }
	gtk_widget_set_usize(entry, 100, 20);
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, 0, 0, 0, 0);
	gtk_table_attach(GTK_TABLE(table), entry, 1, 2, 1, 2, 0, 0, 0, 0);
	gtk_signal_connect(GTK_OBJECT(entry), "changed", GTK_SIGNAL_FUNC(makeprefchanged), NULL);
	gtk_widget_show(label);
        gtk_widget_show(entry);

	label = gtk_label_new("Default From (Route):");
	entry = gtk_entry_new_with_max_length(FRMAX);
	if(strlen(current->From) != 0)
        {
                gtk_entry_set_text(GTK_ENTRY(entry), current->From);
		strncpy(pref->From, current->From, FRMAX);
        }
        else
        {
                gtk_entry_set_text(GTK_ENTRY(entry), "SLC");
        }
	gtk_widget_set_usize(entry, 100, 20);
        gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3, 0, 0, 0, 0);
        gtk_table_attach(GTK_TABLE(table), entry, 1, 2, 2, 3, 0, 0, 0, 0);
	gtk_signal_connect(GTK_OBJECT(entry), "changed", GTK_SIGNAL_FUNC(fromprefchanged), NULL);
        gtk_widget_show(label);
        gtk_widget_show(entry);

        label = gtk_label_new("Default To:");
        entry = gtk_entry_new_with_max_length(TOMAX);
	if(strlen(current->To) != 0)
        {
                gtk_entry_set_text(GTK_ENTRY(entry), current->To);
		strncpy(pref->To, current->To, TOMAX);
        }
        else
        {
                gtk_entry_set_text(GTK_ENTRY(entry), "local");
        }
        gtk_widget_set_usize(entry, 100, 20);
        gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4, 0, 0, 0, 0);
        gtk_table_attach(GTK_TABLE(table), entry, 1, 2, 3, 4, 0, 0, 0, 0);
	gtk_signal_connect(GTK_OBJECT(entry), "changed", GTK_SIGNAL_FUNC(toprefchanged), NULL);
        gtk_widget_show(label);
        gtk_widget_show(entry);

	label = gtk_label_new("Default Type of Approaches:");
	entry = gtk_entry_new_with_max_length(TOAMAX);
	if(strlen(current->TypeOfApproaches) != 0)
        {
                gtk_entry_set_text(GTK_ENTRY(entry), current->TypeOfApproaches);
		strncpy(pref->TypeOfApproaches, current->TypeOfApproaches, TOAMAX);
        }
        else
        {
                gtk_entry_set_text(GTK_ENTRY(entry), "VFR");
        }
        gtk_widget_set_usize(entry, 100, 20);
        gtk_table_attach(GTK_TABLE(table), label, 0, 1, 4, 5, 0, 0, 0, 0);
        gtk_table_attach(GTK_TABLE(table), entry, 1, 2, 4, 5, 0, 0, 0, 0);
	gtk_signal_connect(GTK_OBJECT(entry), "changed", GTK_SIGNAL_FUNC(typeprefchanged), NULL);
        gtk_widget_show(label);
        gtk_widget_show(entry);

/* Start the box for the buttons */
	buttonbox = gtk_hbox_new(FALSE, 3);
	
/* Start the buttons */
	tooltips = gtk_tooltips_new();
	button = gtk_button_new_with_label("Save");
	gtk_box_pack_start(GTK_BOX(buttonbox), button, FALSE, FALSE, 0);
	gtk_widget_show(button);
	gtk_tooltips_set_tip(tooltips, button, "Save the current preferences", NULL);
	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(saveprefs), NULL);

	tooltips = gtk_tooltips_new();	
	button = gtk_button_new_with_label("Close");
	gtk_box_pack_start(GTK_BOX(buttonbox), button, FALSE, FALSE, 0);
	gtk_widget_show(button);
	gtk_tooltips_set_tip(tooltips, button, "Close the current window", NULL);
	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(closeprefwindow), NULL);

	gtk_table_attach(GTK_TABLE(table), buttonbox, 0, 2, 5, 6, 0, 0, 0, 0);

	gtk_container_add(GTK_CONTAINER(prefwindow), table);
	gtk_container_border_width(GTK_CONTAINER(prefwindow), 5);
	gtk_widget_show(buttonbox);
	gtk_widget_show(table);
	gtk_widget_show(prefwindow);

	if (rcfile != NULL)
	{
		fclose(rcfile);
	}
	#ifdef DEBUG
		g_print("Just closed %s.\n", filename);
		fflush(stdout);
	#endif

	free(current);
	#ifdef DEBUG
		g_print("Just freed %i bytes of memory.\n", sizeof(struct CurrentEntries));
		fflush(stdout);
	#endif

}

void closeprefwindow(GtkWidget *widget, gpointer *data)
{
	gtk_widget_destroy(prefwindow);
	prefwindow = NULL;
}

void saveprefs(GtkWidget *widget, gpointer *data)
{
	FILE *rcfile;
	gchar filename[128];

	snprintf(filename, 127, "%s%s", getenv("HOME"), "/.gplbprc");
	rcfile = fopen (filename, "w");
	if (rcfile == NULL)
	{
		GtkWidget *label;	
		GtkWidget *button;
		GtkWidget *vbox;
		GtkTooltips *tooltips;
		gchar error[150];

		if (dialogwindow != NULL)
		{
			gtk_widget_destroy(dialogwindow);
			dialogwindow = NULL;
		}
	
		snprintf(error, 150, "Couldn't open\n%s\nfor writing.", filename);

		vbox = gtk_vbox_new(FALSE, 0);
		
		label = gtk_label_new(error);

		tooltips = gtk_tooltips_new();
		button = gtk_button_new_with_label("Close");
		gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(closedialog), NULL);
		gtk_tooltips_set_tip(tooltips, button, "Close the current window", NULL);

		dialogwindow = gtk_window_new(GTK_WINDOW_DIALOG);
		gtk_window_position(GTK_WINDOW(dialogwindow), GTK_WIN_POS_MOUSE);

		gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
		gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);

		gtk_container_add(GTK_CONTAINER(dialogwindow), vbox);
		gtk_container_border_width(GTK_CONTAINER(dialogwindow), 5);
		gtk_widget_show(label);
		gtk_widget_show(button);
		gtk_widget_show(vbox);
		gtk_widget_show(dialogwindow);
		
	}
	else
	{
		#ifdef DEBUG
			g_print("Just opened %s.\n", filename);
			fflush(stdout);
		#endif
	
		if (strlen(pref->TailNumber) != 0)
		{
			fprintf(rcfile, "tailnumber=%s\n", pref->TailNumber);
		}

	
		if (strlen(pref->From) != 0)
		{
			fprintf(rcfile, "from=%s\n", pref->From);
		}

		if (strlen(pref->To) != 0)
		{
			fprintf(rcfile, "to=%s\n", pref->To);
		}

		if (strlen(pref->MakeModel) != 0)
		{
			fprintf(rcfile, "makemodel=%s\n", pref->MakeModel);
		}
	
		if (strlen(pref->TypeOfApproaches) != 0)
		{
			fprintf(rcfile, "typeofapproaches=%s\n", pref->TypeOfApproaches);
		}
		fclose(rcfile);
		#ifdef DEBUG
			g_print("Just saved data to and closed %s.\n", filename);
			fflush(stdout);
		#endif
	}
	closeprefwindow(NULL, NULL);
}
